@@ -4,12 +4,15 @@ namespace Doctrine\Bundle\DoctrineBundle\Tests\DependencyInjection;
 
 use Doctrine\Bundle\DoctrineBundle\DependencyInjection\Compiler\DbalSchemaFilterPass;
 use Doctrine\Bundle\DoctrineBundle\DependencyInjection\Compiler\EntityListenerPass;
+use Doctrine\Bundle\DoctrineBundle\DependencyInjection\Compiler\WellKnownSchemaFilterPass;
 use Doctrine\Bundle\DoctrineBundle\DependencyInjection\DoctrineExtension;
 use Doctrine\DBAL\Configuration;
 use Doctrine\DBAL\Schema\AbstractAsset;
 use Doctrine\ORM\EntityManager;
 use PHPUnit\Framework\TestCase;
 use Symfony\Bridge\Doctrine\DependencyInjection\CompilerPass\RegisterEventListenersAndSubscribersPass;
+use Symfony\Component\Cache\Adapter\ArrayAdapter;
+use Symfony\Component\Cache\DoctrineProvider;
 use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
 use Symfony\Component\DependencyInjection\Compiler\ResolveChildDefinitionsPass;
 use Symfony\Component\DependencyInjection\ContainerBuilder;
@@ -20,9 +23,9 @@ use Symfony\Component\DependencyInjection\ServiceLocator;
 
 abstract class AbstractDoctrineExtensionTest extends TestCase
 {
-    abstract protected function loadFromFile(ContainerBuilder $container, $file);
+    abstract protected function loadFromFile(ContainerBuilder $container, string $file) : void;
 
-    public function testDbalLoadFromXmlMultipleConnections()
+    public function testDbalLoadFromXmlMultipleConnections() : void
     {
         $container = $this->loadContainer('dbal_service_multiple_connections');
 
@@ -42,6 +45,8 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertSame('sqlite_s3cr3t', $config['password']);
         $this->assertSame('/tmp/db.sqlite', $config['path']);
         $this->assertTrue($config['memory']);
+        $this->assertSame(['asin' => ['callback' => 'asin', 'numArgs' => 1]], $config['driverOptions']['userDefinedFunctions']);
+        $this->assertSame('foo', $config['driverOptions']['arbitraryValue']);
 
         // doctrine.dbal.oci8_connection
         $config = $container->getDefinition('doctrine.dbal.oci_connection')->getArgument(0);
@@ -89,7 +94,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertSame('utf8', $config['charset']);
     }
 
-    public function testDbalLoadFromXmlSingleConnections()
+    public function testDbalLoadFromXmlSingleConnections() : void
     {
         $container = $this->loadContainer('dbal_service_single_connection');
 
@@ -103,7 +108,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertEquals('5.6.20', $config['serverVersion']);
     }
 
-    public function testDbalLoadSingleMasterSlaveConnection()
+    public function testDbalLoadSingleMasterSlaveConnection() : void
     {
         $container = $this->loadContainer('dbal_service_single_master_slave_connection');
 
@@ -137,7 +142,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertEquals(['engine' => 'InnoDB'], $param['defaultTableOptions']);
     }
 
-    public function testDbalLoadPoolShardingConnection()
+    public function testDbalLoadPoolShardingConnection() : void
     {
         $container = $this->loadContainer('dbal_service_pool_sharding_connection');
 
@@ -172,7 +177,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertEquals(['engine' => 'InnoDB'], $param['defaultTableOptions']);
     }
 
-    public function testDbalLoadSavepointsForNestedTransactions()
+    public function testDbalLoadSavepointsForNestedTransactions() : void
     {
         $container = $this->loadContainer('dbal_savepoints');
 
@@ -188,7 +193,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertCount(0, $calls);
     }
 
-    public function testLoadSimpleSingleConnection()
+    public function testLoadSimpleSingleConnection() : void
     {
         $container = $this->loadContainer('orm_service_simple_single_entity_manager');
 
@@ -223,7 +228,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
     /**
      * The PDO driver doesn't require a database name to be to set when connecting to a database server
      */
-    public function testLoadSimpleSingleConnectionWithoutDbName()
+    public function testLoadSimpleSingleConnectionWithoutDbName() : void
     {
         $container = $this->loadContainer('orm_service_simple_single_entity_manager_without_dbname');
 
@@ -258,7 +263,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         ]);
     }
 
-    public function testLoadSingleConnection()
+    public function testLoadSingleConnection() : void
     {
         $container = $this->loadContainer('orm_service_single_entity_manager');
 
@@ -294,7 +299,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($configDef, 'setDefaultRepositoryClassName', ['Acme\Doctrine\Repository']);
     }
 
-    public function testLoadMultipleConnections()
+    public function testLoadMultipleConnections() : void
     {
         $container = $this->loadContainer('orm_service_multiple_entity_managers');
 
@@ -339,16 +344,22 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertEquals('doctrine.orm.em2_configuration', (string) $arguments[1]);
 
         $definition = $container->getDefinition((string) $container->getAlias('doctrine.orm.em1_metadata_cache'));
-        $this->assertEquals('%doctrine_cache.xcache.class%', $definition->getClass());
+        $this->assertEquals(DoctrineProvider::class, $definition->getClass());
 
         $definition = $container->getDefinition((string) $container->getAlias('doctrine.orm.em1_query_cache'));
-        $this->assertEquals('%doctrine_cache.array.class%', $definition->getClass());
+        $this->assertEquals(DoctrineProvider::class, $definition->getClass());
+        $arguments = $definition->getArguments();
+        $this->assertInstanceOf(Reference::class, $arguments[0]);
+        $this->assertEquals('cache.doctrine.orm.em1.query', (string) $arguments[0]);
 
         $definition = $container->getDefinition((string) $container->getAlias('doctrine.orm.em1_result_cache'));
-        $this->assertEquals('%doctrine_cache.array.class%', $definition->getClass());
+        $this->assertEquals(DoctrineProvider::class, $definition->getClass());
+        $arguments = $definition->getArguments();
+        $this->assertInstanceOf(Reference::class, $arguments[0]);
+        $this->assertEquals('cache.doctrine.orm.em1.result', (string) $arguments[0]);
     }
 
-    public function testLoadLogging()
+    public function testLoadLogging() : void
     {
         $container = $this->loadContainer('dbal_logging');
 
@@ -368,67 +379,29 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($definition, 'setSQLLogger', [new Reference('doctrine.dbal.logger.chain.both')]);
     }
 
-    public function testEntityManagerMetadataCacheDriverConfiguration()
+    public function testEntityManagerMetadataCacheDriverConfiguration() : void
     {
         $container = $this->loadContainer('orm_service_multiple_entity_managers');
 
         $definition = $container->getDefinition((string) $container->getAlias('doctrine.orm.em1_metadata_cache'));
-        $this->assertDICDefinitionClass($definition, '%doctrine_cache.xcache.class%');
+        $this->assertDICDefinitionClass($definition, DoctrineProvider::class);
 
         $definition = $container->getDefinition((string) $container->getAlias('doctrine.orm.em2_metadata_cache'));
-        $this->assertDICDefinitionClass($definition, '%doctrine_cache.apc.class%');
+        $this->assertDICDefinitionClass($definition, DoctrineProvider::class);
     }
 
-    public function testEntityManagerMemcacheMetadataCacheDriverConfiguration()
-    {
-        $container = $this->loadContainer('orm_service_simple_single_entity_manager');
-
-        $definition = $container->getDefinition((string) $container->getAlias('doctrine.orm.default_metadata_cache'));
-        $this->assertDICDefinitionClass($definition, '%doctrine_cache.memcache.class%');
-        $this->assertDICDefinitionMethodCallOnce(
-            $definition,
-            'setMemcache',
-            [new Reference('doctrine_cache.services.doctrine.orm.default_metadata_cache.connection')]
-        );
-
-        $definition = $container->getDefinition('doctrine_cache.services.doctrine.orm.default_metadata_cache.connection');
-        $this->assertDICDefinitionClass($definition, '%doctrine_cache.memcache.connection.class%');
-        $this->assertDICDefinitionMethodCallOnce($definition, 'addServer', [
-            'localhost',
-            '11211',
-        ]);
-    }
-
-    public function testEntityManagerRedisMetadataCacheDriverConfigurationWithDatabaseKey()
-    {
-        $container = $this->loadContainer('orm_service_simple_single_entity_manager_redis');
-
-        $definition = $container->getDefinition((string) $container->getAlias('doctrine.orm.default_metadata_cache'));
-        $this->assertDICDefinitionClass($definition, '%doctrine_cache.redis.class%');
-        $this->assertDICDefinitionMethodCallOnce(
-            $definition,
-            'setRedis',
-            [new Reference('doctrine_cache.services.doctrine.orm.default_metadata_cache_redis.connection')]
-        );
-
-        $definition = $container->getDefinition('doctrine_cache.services.doctrine.orm.default_metadata_cache_redis.connection');
-        $this->assertDICDefinitionClass($definition, '%doctrine_cache.redis.connection.class%');
-        $this->assertDICDefinitionMethodCallOnce($definition, 'connect', ['localhost', '6379']);
-        $this->assertDICDefinitionMethodCallOnce($definition, 'select', [1]);
-    }
-
-    public function testDependencyInjectionImportsOverrideDefaults()
+    public function testDependencyInjectionImportsOverrideDefaults() : void
     {
         $container = $this->loadContainer('orm_imports');
 
         $cacheDefinition = $container->getDefinition((string) $container->getAlias('doctrine.orm.default_metadata_cache'));
-        $this->assertEquals('%doctrine_cache.apc.class%', $cacheDefinition->getClass());
+        $this->assertEquals(DoctrineProvider::class, $cacheDefinition->getClass());
 
         $configDefinition = $container->getDefinition('doctrine.orm.default_configuration');
         $this->assertDICDefinitionMethodCallOnce($configDefinition, 'setAutoGenerateProxyClasses', ['%doctrine.orm.auto_generate_proxy_classes%']);
     }
 
-    public function testSingleEntityManagerMultipleMappingBundleDefinitions()
+    public function testSingleEntityManagerMultipleMappingBundleDefinitions() : void
     {
         $container = $this->loadContainer('orm_single_em_bundle_mappings', ['YamlBundle', 'AnnotationsBundle', 'XmlBundle']);
 
@@ -466,7 +439,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         ]);
     }
 
-    public function testMultipleEntityManagersMappingBundleDefinitions()
+    public function testMultipleEntityManagersMappingBundleDefinitions() : void
     {
         $container = $this->loadContainer('orm_multiple_em_bundle_mappings', ['YamlBundle', 'AnnotationsBundle', 'XmlBundle']);
 
@@ -508,7 +481,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         ]);
     }
 
-    public function testSingleEntityManagerDefaultTableOptions()
+    public function testSingleEntityManagerDefaultTableOptions() : void
     {
         $container = $this->loadContainer('orm_single_em_default_table_options', ['YamlBundle', 'AnnotationsBundle', 'XmlBundle']);
 
@@ -527,18 +500,18 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertEquals('InnoDB', $defaults['engine']);
     }
 
-    public function testSetTypes()
+    public function testSetTypes() : void
     {
         $container = $this->loadContainer('dbal_types');
 
         $this->assertEquals(
-            ['test' => ['class' => TestType::class, 'commented' => null]],
+            ['test' => ['class' => TestType::class]],
             $container->getParameter('doctrine.dbal.connection_factory.types')
         );
         $this->assertEquals('%doctrine.dbal.connection_factory.types%', $container->getDefinition('doctrine.dbal.connection_factory')->getArgument(0));
     }
 
-    public function testSetCustomFunctions()
+    public function testSetCustomFunctions() : void
     {
         $container = $this->loadContainer('orm_functions');
 
@@ -548,7 +521,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($definition, 'addCustomDatetimeFunction', ['test_datetime', 'Symfony\Bundle\DoctrineBundle\Tests\DependencyInjection\TestDatetimeFunction']);
     }
 
-    public function testSetNamingStrategy()
+    public function testSetNamingStrategy() : void
     {
         $container = $this->loadContainer('orm_namingstrategy');
 
@@ -559,7 +532,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($def2, 'setNamingStrategy', [0 => new Reference('doctrine.orm.naming_strategy.underscore')]);
     }
 
-    public function testSetQuoteStrategy()
+    public function testSetQuoteStrategy() : void
     {
         $container = $this->loadContainer('orm_quotestrategy');
 
@@ -570,7 +543,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($def2, 'setQuoteStrategy', [0 => new Reference('doctrine.orm.quote_strategy.ansi')]);
     }
 
-    public function testSecondLevelCache()
+    public function testSecondLevelCache() : void
     {
         $container = $this->loadContainer('orm_second_level_cache');
 
@@ -605,7 +578,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionClass($myEntityRegionDef, '%doctrine.orm.second_level_cache.default_region.class%');
         $this->assertDICDefinitionClass($loggerChainDef, '%doctrine.orm.second_level_cache.logger_chain.class%');
         $this->assertDICDefinitionClass($loggerStatisticsDef, '%doctrine.orm.second_level_cache.logger_statistics.class%');
-        $this->assertDICDefinitionClass($cacheDriverDef, '%doctrine_cache.array.class%');
+        $this->assertDICDefinitionClass($cacheDriverDef, DoctrineProvider::class);
         $this->assertDICDefinitionMethodCallOnce($configDef, 'setSecondLevelCacheConfiguration');
         $this->assertDICDefinitionMethodCallCount($slcFactoryDef, 'setRegion', [], 3);
         $this->assertDICDefinitionMethodCallCount($loggerChainDef, 'setLogger', [], 3);
@@ -628,7 +601,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertEquals('doctrine.orm.default_second_level_cache.region_cache_driver', $slcFactoryArgs[1]);
     }
 
-    public function testSingleEMSetCustomFunctions()
+    public function testSingleEMSetCustomFunctions() : void
     {
         $container = $this->loadContainer('orm_single_em_dql_functions');
 
@@ -636,7 +609,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($definition, 'addCustomStringFunction', ['test_string', 'Symfony\Bundle\DoctrineBundle\Tests\DependencyInjection\TestStringFunction']);
     }
 
-    public function testAddCustomHydrationMode()
+    public function testAddCustomHydrationMode() : void
     {
         $container = $this->loadContainer('orm_hydration_mode');
 
@@ -644,7 +617,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($definition, 'addCustomHydrationMode', ['test_hydrator', 'Symfony\Bundle\DoctrineBundle\Tests\DependencyInjection\TestHydrator']);
     }
 
-    public function testAddFilter()
+    public function testAddFilter() : void
     {
         $container = $this->loadContainer('orm_filters');
 
@@ -664,7 +637,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertCount(2, $entityManager->getFilters()->getEnabledFilters());
     }
 
-    public function testResolveTargetEntity()
+    public function testResolveTargetEntity() : void
     {
         $container = $this->loadContainer('orm_resolve_target_entity');
 
@@ -674,7 +647,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertEquals(['doctrine.event_subscriber' => [[]]], $definition->getTags());
     }
 
-    public function testAttachEntityListeners()
+    public function testAttachEntityListeners() : void
     {
         $container = $this->loadContainer('orm_attach_entity_listener');
 
@@ -745,7 +718,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         ]);
     }
 
-    public function testDbalAutoCommit()
+    public function testDbalAutoCommit() : void
     {
         $container = $this->loadContainer('dbal_auto_commit');
 
@@ -753,7 +726,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($definition, 'setAutoCommit', [false]);
     }
 
-    public function testDbalOracleConnectstring()
+    public function testDbalOracleConnectstring() : void
     {
         $container = $this->loadContainer('dbal_oracle_connectstring');
 
@@ -761,7 +734,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertSame('scott@sales-server:1521/sales.us.example.com', $config['connectstring']);
     }
 
-    public function testDbalOracleInstancename()
+    public function testDbalOracleInstancename() : void
     {
         $container = $this->loadContainer('dbal_oracle_instancename');
 
@@ -769,27 +742,12 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertSame('mySuperInstance', $config['instancename']);
     }
 
-    public function testDbalSchemaFilter()
+    public function testDbalSchemaFilterNewConfig() : void
     {
-        if (method_exists(Configuration::class, 'setSchemaAssetsFilter')) {
-            $this->markTestSkipped('Test only applies to doctrine/dbal 2.8 or lower');
-        }
-
-        $container = $this->loadContainer('dbal_schema_filter');
-
-        $definition = $container->getDefinition('doctrine.dbal.connection1_connection.configuration');
-        $this->assertDICDefinitionMethodCallOnce($definition, 'setFilterSchemaAssetsExpression', ['~^(?!t_)~']);
-    }
-
-    public function testDbalSchemaFilterNewConfig()
-    {
-        if (! method_exists(Configuration::class, 'setSchemaAssetsFilter')) {
-            $this->markTestSkipped('Test requires doctrine/dbal 2.9 or higher');
-        }
-
         $container = $this->getContainer([]);
         $loader    = new DoctrineExtension();
         $container->registerExtension($loader);
+        $container->addCompilerPass(new WellKnownSchemaFilterPass());
         $container->addCompilerPass(new DbalSchemaFilterPass());
 
         // ignore table1 table on "default" connection
@@ -827,7 +785,73 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertNull($connConfig = $getConfiguration('connection3')->getSchemaAssetsFilter());
     }
 
-    public function testEntityListenerResolver()
+    public static function dataWellKnownSchemaFilterServices()
+    {
+        yield ['cache', 'cache_items'];
+        yield ['lock', 'lock_keys'];
+        yield ['messenger', 'messenger_messages'];
+        yield ['session', 'sessions'];
+    }
+
+    /**
+     * @dataProvider dataWellKnownSchemaFilterServices
+     */
+    public function testWellKnownSchemaFilterDefaultTables(string $fileName, string $tableName) : void
+    {
+        $container = $this->getContainer([]);
+        $loader    = new DoctrineExtension();
+        $container->registerExtension($loader);
+        $container->addCompilerPass(new WellKnownSchemaFilterPass());
+        $container->addCompilerPass(new DbalSchemaFilterPass());
+
+        $this->loadFromFile($container, 'well_known_schema_filter_default_tables_' . $fileName);
+
+        $this->compileContainer($container);
+
+        $definition = $container->getDefinition('doctrine.dbal.well_known_schema_asset_filter');
+
+        $this->assertSame([[$tableName]], $definition->getArguments());
+        $this->assertSame([['connection' => 'connection1'], ['connection' => 'connection2'], ['connection' => 'connection3']], $definition->getTag('doctrine.dbal.schema_filter'));
+
+        $definition = $container->getDefinition('doctrine.dbal.connection1_schema_asset_filter_manager');
+
+        $this->assertEquals([new Reference('doctrine.dbal.well_known_schema_asset_filter'), new Reference('doctrine.dbal.connection1_regex_schema_filter')], $definition->getArgument(0));
+
+        $filter = $container->get('well_known_filter');
+
+        $this->assertFalse($filter($tableName));
+        $this->assertTrue($filter('anything_else'));
+    }
+
+    public static function dataWellKnownSchemaOverriddenTablesFilterServices()
+    {
+        yield ['cache', 'app_cache'];
+        yield ['lock', 'app_locks'];
+        yield ['messenger', 'app_messages'];
+        yield ['session', 'app_session'];
+    }
+
+    /**
+     * @dataProvider dataWellKnownSchemaOverriddenTablesFilterServices
+     */
+    public function testWellKnownSchemaFilterOverriddenTables(string $fileName, string $tableName) : void
+    {
+        $container = $this->getContainer([]);
+        $loader    = new DoctrineExtension();
+        $container->registerExtension($loader);
+        $container->addCompilerPass(new WellKnownSchemaFilterPass());
+        $container->addCompilerPass(new DbalSchemaFilterPass());
+
+        $this->loadFromFile($container, 'well_known_schema_filter_overridden_tables_' . $fileName);
+
+        $this->compileContainer($container);
+
+        $filter = $container->get('well_known_filter');
+
+        $this->assertFalse($filter($tableName));
+    }
+
+    public function testEntityListenerResolver() : void
     {
         $container = $this->loadContainer('orm_entity_listener_resolver', ['YamlBundle'], new EntityListenerPass());
 
@@ -844,7 +868,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($listener, 'register', [new Reference('entity_listener2')]);
     }
 
-    public function testAttachEntityListenerTag()
+    public function testAttachEntityListenerTag() : void
     {
         $container = $this->getContainer([]);
         $loader    = new DoctrineExtension();
@@ -856,19 +880,27 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->compileContainer($container);
 
         $listener = $container->getDefinition('doctrine.orm.em1_entity_listener_resolver');
-        $this->assertDICDefinitionMethodCallOnce($listener, 'registerService', ['EntityListener1', 'entity_listener1']);
+        $this->assertDICDefinitionMethodCallCount($listener, 'registerService', [
+            ['EntityListener1', 'entity_listener1'],
+            ['Doctrine\Bundle\DoctrineBundle\Tests\DependencyInjection\Fixtures\InvokableEntityListener', 'invokable_entity_listener'],
+            ['Doctrine\Bundle\DoctrineBundle\Tests\DependencyInjection\Fixtures\InvokableEntityListener', 'invokable_entity_listener'],
+            ['ParentEntityListener', 'children_entity_listener'],
+        ], 4);
 
         $listener = $container->getDefinition('doctrine.orm.em2_entity_listener_resolver');
         $this->assertDICDefinitionMethodCallOnce($listener, 'registerService', ['EntityListener2', 'entity_listener2']);
 
         $attachListener = $container->getDefinition('doctrine.orm.em1_listeners.attach_entity_listeners');
-        $this->assertDICDefinitionMethodCallOnce($attachListener, 'addEntityListener', ['My/Entity1', 'EntityListener1', 'postLoad']);
+        $this->assertDICDefinitionMethodCallAt(0, $attachListener, 'addEntityListener', ['My/Entity1', 'EntityListener1', 'postLoad']);
+        $this->assertDICDefinitionMethodCallAt(1, $attachListener, 'addEntityListener', ['My/Entity1', 'Doctrine\Bundle\DoctrineBundle\Tests\DependencyInjection\Fixtures\InvokableEntityListener', 'loadClassMetadata', '__invoke']);
+        $this->assertDICDefinitionMethodCallAt(2, $attachListener, 'addEntityListener', ['My/Entity1', 'Doctrine\Bundle\DoctrineBundle\Tests\DependencyInjection\Fixtures\InvokableEntityListener', 'postPersist']);
+        $this->assertDICDefinitionMethodCallAt(3, $attachListener, 'addEntityListener', ['My/Entity3', 'ParentEntityListener', 'postLoad']);
 
         $attachListener = $container->getDefinition('doctrine.orm.em2_listeners.attach_entity_listeners');
         $this->assertDICDefinitionMethodCallOnce($attachListener, 'addEntityListener', ['My/Entity2', 'EntityListener2', 'preFlush', 'preFlushHandler']);
     }
 
-    public function testAttachEntityListenersTwoConnections()
+    public function testAttachEntityListenersTwoConnections() : void
     {
         $container = $this->getContainer(['YamlBundle']);
         $loader    = new DoctrineExtension();
@@ -888,7 +920,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($foobarEventManager, 'addEventListener', [['loadClassMetadata'], new Reference('doctrine.orm.em2_listeners.attach_entity_listeners')]);
     }
 
-    public function testAttachLazyEntityListener()
+    public function testAttachLazyEntityListener() : void
     {
         $container = $this->getContainer([]);
         $loader    = new DoctrineExtension();
@@ -915,7 +947,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($resolver2, 'registerService', ['EntityListener2', 'entity_listener2']);
     }
 
-    public function testAttachLazyEntityListenerForCustomResolver()
+    public function testAttachLazyEntityListenerForCustomResolver() : void
     {
         $container = $this->getContainer([]);
         $loader    = new DoctrineExtension();
@@ -937,7 +969,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
      * @expectedException \InvalidArgumentException
      * @expectedExceptionMessage EntityListenerServiceResolver
      */
-    public function testLazyEntityListenerResolverWithoutCorrectInterface()
+    public function testLazyEntityListenerResolverWithoutCorrectInterface() : void
     {
         $container = $this->getContainer([]);
         $loader    = new DoctrineExtension();
@@ -949,7 +981,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->compileContainer($container);
     }
 
-    public function testPrivateLazyEntityListener()
+    public function testPrivateLazyEntityListener() : void
     {
         $container = $this->getContainer([]);
         $loader    = new DoctrineExtension();
@@ -967,7 +999,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
      * @expectedException \InvalidArgumentException
      * @expectedExceptionMessageRegExp /The service ".*" must not be abstract as this entity listener is lazy-loaded/
      */
-    public function testAbstractLazyEntityListener()
+    public function testAbstractLazyEntityListener() : void
     {
         $container = $this->getContainer([]);
         $loader    = new DoctrineExtension();
@@ -979,7 +1011,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->compileContainer($container);
     }
 
-    public function testRepositoryFactory()
+    public function testRepositoryFactory() : void
     {
         $container = $this->loadContainer('orm_repository_factory');
 
@@ -987,8 +1019,11 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->assertDICDefinitionMethodCallOnce($definition, 'setRepositoryFactory', ['repository_factory']);
     }
 
-    private function loadContainer($fixture, array $bundles = ['YamlBundle'], CompilerPassInterface $compilerPass = null)
-    {
+    private function loadContainer(
+        string $fixture,
+        array $bundles = ['YamlBundle'],
+        CompilerPassInterface $compilerPass = null
+    ) : ContainerBuilder {
         $container = $this->getContainer($bundles);
         $container->registerExtension(new DoctrineExtension());
 
@@ -1003,7 +1038,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         return $container;
     }
 
-    private function getContainer(array $bundles)
+    private function getContainer(array $bundles) : ContainerBuilder
     {
         $map = [];
         foreach ($bundles as $bundle) {
@@ -1012,33 +1047,44 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
             $map[$bundle] = 'Fixtures\\Bundles\\' . $bundle . '\\' . $bundle;
         }
 
-        return new ContainerBuilder(new ParameterBag([
+        $container = new ContainerBuilder(new ParameterBag([
             'kernel.name' => 'app',
             'kernel.debug' => false,
             'kernel.bundles' => $map,
             'kernel.cache_dir' => sys_get_temp_dir(),
             'kernel.environment' => 'test',
             'kernel.root_dir' => __DIR__ . '/../../', // src dir
+            'kernel.project_dir' => __DIR__ . '/../../', // src dir
+            'kernel.bundles_metadata' => [],
+            'container.build_id' => uniqid(),
         ]));
+
+        // Register dummy cache services so we don't have to load the FrameworkExtension
+        $container->setDefinition('cache.system', (new Definition(ArrayAdapter::class))->setPublic(true));
+        $container->setDefinition('cache.app', (new Definition(ArrayAdapter::class))->setPublic(true));
+
+        return $container;
     }
 
     /**
      * Assertion on the Class of a DIC Service Definition.
-     *
-     * @param string $expectedClass
      */
-    private function assertDICDefinitionClass(Definition $definition, $expectedClass)
+    private function assertDICDefinitionClass(Definition $definition, string $expectedClass) : void
     {
         $this->assertEquals($expectedClass, $definition->getClass(), 'Expected Class of the DIC Container Service Definition is wrong.');
     }
 
-    private function assertDICConstructorArguments(Definition $definition, $args)
+    private function assertDICConstructorArguments(Definition $definition, array $args) : void
     {
         $this->assertEquals($args, $definition->getArguments(), "Expected and actual DIC Service constructor arguments of definition '" . $definition->getClass() . "' don't match.");
     }
 
-    private function assertDICDefinitionMethodCallAt($pos, Definition $definition, $methodName, array $params = null)
-    {
+    private function assertDICDefinitionMethodCallAt(
+        int $pos,
+        Definition $definition,
+        string $methodName,
+        array $params = null
+    ) : void {
         $calls = $definition->getMethodCalls();
         if (! isset($calls[$pos][0])) {
             $this->fail(sprintf('Method call at position %s not found!', $pos));
@@ -1057,12 +1103,12 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
 
     /**
      * Assertion for the DI Container, check if the given definition contains a method call with the given parameters.
-     *
-     * @param string $methodName
-     * @param array  $params
      */
-    private function assertDICDefinitionMethodCallOnce(Definition $definition, $methodName, array $params = null)
-    {
+    private function assertDICDefinitionMethodCallOnce(
+        Definition $definition,
+        string $methodName,
+        array $params = null
+    ) : void {
         $calls  = $definition->getMethodCalls();
         $called = false;
         foreach ($calls as $call) {
@@ -1086,8 +1132,12 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         $this->fail("Method '" . $methodName . "' is expected to be called once, definition does not contain a call though.");
     }
 
-    private function assertDICDefinitionMethodCallCount(Definition $definition, $methodName, array $params = [], $nbCalls = 1)
-    {
+    private function assertDICDefinitionMethodCallCount(
+        Definition $definition,
+        string $methodName,
+        array $params = [],
+        int $nbCalls = 1
+    ) : void {
         $calls  = $definition->getMethodCalls();
         $called = 0;
         foreach ($calls as $call) {
@@ -1110,12 +1160,12 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
 
     /**
      * Assertion for the DI Container, check if the given definition does not contain a method call with the given parameters.
-     *
-     * @param string $methodName
-     * @param array  $params
      */
-    private function assertDICDefinitionNoMethodCall(Definition $definition, $methodName, array $params = null)
-    {
+    private function assertDICDefinitionNoMethodCall(
+        Definition $definition,
+        string $methodName,
+        array $params = null
+    ) : void {
         $calls = $definition->getMethodCalls();
         foreach ($calls as $call) {
             if ($call[0] !== $methodName) {
@@ -1130,7 +1180,7 @@ abstract class AbstractDoctrineExtensionTest extends TestCase
         }
     }
 
-    private function compileContainer(ContainerBuilder $container)
+    private function compileContainer(ContainerBuilder $container) : void
     {
         $container->getCompilerPassConfig()->setOptimizationPasses([new ResolveChildDefinitionsPass()]);
         $container->getCompilerPassConfig()->setRemovingPasses([]);
