@@ -4,13 +4,14 @@ namespace Doctrine\Bundle\DoctrineBundle\Tests\DependencyInjection;
 
 use Doctrine\Bundle\DoctrineBundle\DependencyInjection\DoctrineExtension;
 use Doctrine\Bundle\DoctrineBundle\Tests\Builder\BundleConfigurationBuilder;
-use Doctrine\Common\Persistence\ManagerRegistry;
-use Doctrine\Common\Persistence\ObjectManager;
-use Doctrine\Common\Proxy\AbstractProxyFactory;
 use Doctrine\DBAL\Connection;
 use Doctrine\DBAL\Driver\Connection as DriverConnection;
 use Doctrine\ORM\EntityManagerInterface;
+use InvalidArgumentException;
 use PHPUnit\Framework\TestCase;
+use Symfony\Bridge\Doctrine\Messenger\DoctrineClearEntityManagerWorkerSubscriber;
+use Symfony\Component\Cache\Adapter\ArrayAdapter;
+use Symfony\Component\Cache\DoctrineProvider;
 use Symfony\Component\DependencyInjection\Compiler\ResolveChildDefinitionsPass;
 use Symfony\Component\DependencyInjection\ContainerBuilder;
 use Symfony\Component\DependencyInjection\Definition;
@@ -20,7 +21,12 @@ use Symfony\Component\Messenger\MessageBusInterface;
 
 class DoctrineExtensionTest extends TestCase
 {
-    public function testAutowiringAlias()
+    /**
+     * https://github.com/doctrine/orm/pull/7953 needed, otherwise ORM classes we define services for trigger deprecations
+     *
+     * @group legacy
+     */
+    public function testAutowiringAlias() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -31,8 +37,6 @@ class DoctrineExtensionTest extends TestCase
         $expectedAliases = [
             DriverConnection::class => 'database_connection',
             Connection::class => 'database_connection',
-            ManagerRegistry::class => 'doctrine',
-            ObjectManager::class => 'doctrine.orm.entity_manager',
             EntityManagerInterface::class => 'doctrine.orm.entity_manager',
         ];
 
@@ -45,7 +49,7 @@ class DoctrineExtensionTest extends TestCase
         }
     }
 
-    public function testPublicServicesAndAliases()
+    public function testPublicServicesAndAliases() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -58,7 +62,7 @@ class DoctrineExtensionTest extends TestCase
         $this->assertTrue($container->getAlias('database_connection')->isPublic());
     }
 
-    public function testDbalGenerateDefaultConnectionConfiguration()
+    public function testDbalGenerateDefaultConnectionConfiguration() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -78,7 +82,7 @@ class DoctrineExtensionTest extends TestCase
         $this->assertEquals([], $container->getDefinition('doctrine.dbal.default_connection')->getArgument(0)['driverOptions']);
     }
 
-    public function testDbalOverrideDefaultConnection()
+    public function testDbalOverrideDefaultConnection() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -96,14 +100,14 @@ class DoctrineExtensionTest extends TestCase
      * @expectedException \LogicException
      * @expectedExceptionMessage Configuring the ORM layer requires to configure the DBAL layer as well.
      */
-    public function testOrmRequiresDbal()
+    public function testOrmRequiresDbal() : void
     {
         $extension = new DoctrineExtension();
 
         $extension->load([['orm' => ['auto_mapping' => true]]], $this->getContainer());
     }
 
-    public function getAutomappingConfigurations()
+    public function getAutomappingConfigurations() : array
     {
         return [
             [
@@ -141,7 +145,7 @@ class DoctrineExtensionTest extends TestCase
     /**
      * @dataProvider getAutomappingConfigurations
      */
-    public function testAutomapping(array $entityManagers)
+    public function testAutomapping(array $entityManagers) : void
     {
         $extension = new DoctrineExtension();
 
@@ -190,7 +194,7 @@ class DoctrineExtensionTest extends TestCase
         );
     }
 
-    public function testDbalLoad()
+    public function testDbalLoad() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -208,7 +212,7 @@ class DoctrineExtensionTest extends TestCase
         $this->assertEquals('root', $config['user']);
     }
 
-    public function testDbalWrapperClass()
+    public function testDbalWrapperClass() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -234,7 +238,7 @@ class DoctrineExtensionTest extends TestCase
         $this->assertNull($container->getDefinition('doctrine.dbal.second_connection')->getClass());
     }
 
-    public function testDependencyInjectionConfigurationDefaults()
+    public function testDependencyInjectionConfigurationDefaults() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -320,16 +324,28 @@ class DoctrineExtensionTest extends TestCase
         $this->assertEquals('doctrine.orm.default_entity_listener_resolver', (string) $calls[12][1][0]);
 
         $definition = $container->getDefinition((string) $container->getAlias('doctrine.orm.default_metadata_cache'));
-        $this->assertEquals('%doctrine_cache.array.class%', $definition->getClass());
+        $this->assertEquals(DoctrineProvider::class, $definition->getClass());
+        $arguments = $definition->getArguments();
+        $this->assertInstanceOf(Reference::class, $arguments[0]);
+        $this->assertEquals('cache.doctrine.orm.default.metadata', (string) $arguments[0]);
+        $this->assertSame(ArrayAdapter::class, $container->getDefinition((string) $arguments[0])->getClass());
 
         $definition = $container->getDefinition((string) $container->getAlias('doctrine.orm.default_query_cache'));
-        $this->assertEquals('%doctrine_cache.array.class%', $definition->getClass());
+        $this->assertEquals(DoctrineProvider::class, $definition->getClass());
+        $arguments = $definition->getArguments();
+        $this->assertInstanceOf(Reference::class, $arguments[0]);
+        $this->assertEquals('cache.doctrine.orm.default.query', (string) $arguments[0]);
+        $this->assertSame(ArrayAdapter::class, $container->getDefinition((string) $arguments[0])->getClass());
 
         $definition = $container->getDefinition((string) $container->getAlias('doctrine.orm.default_result_cache'));
-        $this->assertEquals('%doctrine_cache.array.class%', $definition->getClass());
+        $this->assertEquals(DoctrineProvider::class, $definition->getClass());
+        $arguments = $definition->getArguments();
+        $this->assertInstanceOf(Reference::class, $arguments[0]);
+        $this->assertEquals('cache.doctrine.orm.default.result', (string) $arguments[0]);
+        $this->assertSame(ArrayAdapter::class, $container->getDefinition((string) $arguments[0])->getClass());
     }
 
-    public function testUseSavePointsAddMethodCallToAddSavepointsToTheConnection()
+    public function testUseSavePointsAddMethodCallToAddSavepointsToTheConnection() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -349,7 +365,7 @@ class DoctrineExtensionTest extends TestCase
         $this->assertTrue($calls[0][1][0]);
     }
 
-    public function testAutoGenerateProxyClasses()
+    public function testAutoGenerateProxyClasses() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -370,10 +386,10 @@ class DoctrineExtensionTest extends TestCase
 
         $extension->load([$config], $container);
 
-        $this->assertEquals(AbstractProxyFactory::AUTOGENERATE_EVAL, $container->getParameter('doctrine.orm.auto_generate_proxy_classes'));
+        $this->assertEquals(3 /* \Doctrine\Common\Proxy\AbstractProxyFactory::AUTOGENERATE_EVAL */, $container->getParameter('doctrine.orm.auto_generate_proxy_classes'));
     }
 
-    public function testSingleEntityManagerWithDefaultConfiguration()
+    public function testSingleEntityManagerWithDefaultConfiguration() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -393,7 +409,7 @@ class DoctrineExtensionTest extends TestCase
         ]);
     }
 
-    public function testSingleEntityManagerWithDefaultSecondLevelCacheConfiguration()
+    public function testSingleEntityManagerWithDefaultSecondLevelCacheConfiguration() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -418,14 +434,14 @@ class DoctrineExtensionTest extends TestCase
         $this->assertEquals('%doctrine.orm.second_level_cache.default_cache_factory.class%', $slcDefinition->getClass());
     }
 
-    public function testSingleEntityManagerWithCustomSecondLevelCacheConfiguration()
+    public function testSingleEntityManagerWithCustomSecondLevelCacheConfiguration() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
 
         $configurationArray = BundleConfigurationBuilder::createBuilderWithBaseValues()
             ->addSecondLevelCache([
-                'region_cache_driver' => ['type' => 'memcache'],
+                'region_cache_driver' => ['type' => 'service', 'id' => 'my_cache'],
                 'regions' => [
                     'hour_region' => ['lifetime' => 3600],
                 ],
@@ -449,7 +465,7 @@ class DoctrineExtensionTest extends TestCase
         $this->assertEquals('YamlBundle\Cache\MyCacheFactory', $slcDefinition->getClass());
     }
 
-    public function testBundleEntityAliases()
+    public function testBundleEntityAliases() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -468,7 +484,7 @@ class DoctrineExtensionTest extends TestCase
         );
     }
 
-    public function testOverwriteEntityAliases()
+    public function testOverwriteEntityAliases() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -487,7 +503,7 @@ class DoctrineExtensionTest extends TestCase
         );
     }
 
-    public function testYamlBundleMappingDetection()
+    public function testYamlBundleMappingDetection() : void
     {
         $container = $this->getContainer('YamlBundle');
         $extension = new DoctrineExtension();
@@ -505,7 +521,7 @@ class DoctrineExtensionTest extends TestCase
         ]);
     }
 
-    public function testXmlBundleMappingDetection()
+    public function testXmlBundleMappingDetection() : void
     {
         $container = $this->getContainer('XmlBundle');
         $extension = new DoctrineExtension();
@@ -532,7 +548,7 @@ class DoctrineExtensionTest extends TestCase
         ]);
     }
 
-    public function testAnnotationsBundleMappingDetection()
+    public function testAnnotationsBundleMappingDetection() : void
     {
         $container = $this->getContainer('AnnotationsBundle');
         $extension = new DoctrineExtension();
@@ -559,7 +575,7 @@ class DoctrineExtensionTest extends TestCase
         ]);
     }
 
-    public function testOrmMergeConfigs()
+    public function testOrmMergeConfigs() : void
     {
         $container = $this->getContainer(['XmlBundle', 'AnnotationsBundle']);
         $extension = new DoctrineExtension();
@@ -616,7 +632,7 @@ class DoctrineExtensionTest extends TestCase
         }
     }
 
-    public function testAnnotationsBundleMappingDetectionWithVendorNamespace()
+    public function testAnnotationsBundleMappingDetectionWithVendorNamespace() : void
     {
         $container = $this->getContainer('AnnotationsBundle', 'Vendor');
         $extension = new DoctrineExtension();
@@ -641,7 +657,7 @@ class DoctrineExtensionTest extends TestCase
         $this->assertEquals('Fixtures\Bundles\Vendor\AnnotationsBundle\Entity', $calls[0][1][1]);
     }
 
-    public function testMessengerIntegration()
+    public function testMessengerIntegration() : void
     {
         if (! interface_exists(MessageBusInterface::class)) {
             $this->markTestSkipped('Symfony Messenger component is not installed');
@@ -652,12 +668,6 @@ class DoctrineExtensionTest extends TestCase
 
         $config = BundleConfigurationBuilder::createBuilder()
             ->addBaseConnection()
-            ->addEntityManager([
-                'default_entity_manager' => 'default',
-                'entity_managers' => [
-                    'default' => [],
-                ],
-            ])
             ->build();
         $extension->load([$config], $container);
 
@@ -667,6 +677,29 @@ class DoctrineExtensionTest extends TestCase
         $this->assertCount(1, $middlewarePrototype->getArguments());
         $this->assertNotNull($middlewarePrototype = $container->getDefinition('messenger.middleware.doctrine_close_connection'));
         $this->assertCount(1, $middlewarePrototype->getArguments());
+
+        if (class_exists(DoctrineClearEntityManagerWorkerSubscriber::class)) {
+            $this->assertNotNull($subscriber = $container->getDefinition('doctrine.orm.messenger.event_subscriber.doctrine_clear_entity_manager'));
+            $this->assertCount(1, $subscriber->getArguments());
+        } else {
+            $this->assertFalse($container->hasDefinition('doctrine.orm.messenger.event_subscriber.doctrine_clear_entity_manager'));
+        }
+    }
+
+    public function testInvalidCacheConfiguration() : void
+    {
+        $container = $this->getContainer();
+        $extension = new DoctrineExtension();
+
+        $config = BundleConfigurationBuilder::createBuilder()
+            ->addBaseConnection()
+            ->addEntityManager(['metadata_cache_driver' => 'redis'])
+            ->build();
+
+        $this->expectException(InvalidArgumentException::class);
+        $this->expectExceptionMessage('Unknown cache of type "redis" configured for cache "metadata_cache" in entity manager "default"');
+
+        $extension->load([$config], $container);
     }
 
     /**
@@ -694,23 +727,42 @@ class DoctrineExtensionTest extends TestCase
     public static function cacheConfigurationProvider() : array
     {
         return [
-            'metadata_cache_provider' => [
+            'metadata_cache_default' => [
                 'expectedAliasName' => 'doctrine.orm.default_metadata_cache',
-                'expectedAliasTarget' => 'doctrine_cache.providers.metadata_cache',
+                'expectedAliasTarget' => 'doctrine.orm.cache.provider.cache.doctrine.orm.default.metadata',
                 'cacheName' => 'metadata_cache_driver',
-                'cacheConfig' => ['cache_provider' => 'metadata_cache'],
+                'cacheConfig' => ['type' => null],
             ],
-            'query_cache_provider' => [
+            'query_cache_default' => [
                 'expectedAliasName' => 'doctrine.orm.default_query_cache',
-                'expectedAliasTarget' => 'doctrine_cache.providers.query_cache',
+                'expectedAliasTarget' => 'doctrine.orm.cache.provider.cache.doctrine.orm.default.query',
                 'cacheName' => 'query_cache_driver',
-                'cacheConfig' => ['cache_provider' => 'query_cache'],
+                'cacheConfig' => ['type' => null],
             ],
-            'result_cache_provider' => [
+            'result_cache_default' => [
                 'expectedAliasName' => 'doctrine.orm.default_result_cache',
-                'expectedAliasTarget' => 'doctrine_cache.providers.result_cache',
+                'expectedAliasTarget' => 'doctrine.orm.cache.provider.cache.doctrine.orm.default.result',
                 'cacheName' => 'result_cache_driver',
-                'cacheConfig' => ['cache_provider' => 'result_cache'],
+                'cacheConfig' => ['type' => null],
+            ],
+
+            'metadata_cache_pool' => [
+                'expectedAliasName' => 'doctrine.orm.default_metadata_cache',
+                'expectedAliasTarget' => 'doctrine.orm.cache.provider.metadata_cache_pool',
+                'cacheName' => 'metadata_cache_driver',
+                'cacheConfig' => ['type' => 'pool', 'pool' => 'metadata_cache_pool'],
+            ],
+            'query_cache_pool' => [
+                'expectedAliasName' => 'doctrine.orm.default_query_cache',
+                'expectedAliasTarget' => 'doctrine.orm.cache.provider.query_cache_pool',
+                'cacheName' => 'query_cache_driver',
+                'cacheConfig' => ['type' => 'pool', 'pool' => 'query_cache_pool'],
+            ],
+            'result_cache_pool' => [
+                'expectedAliasName' => 'doctrine.orm.default_result_cache',
+                'expectedAliasTarget' => 'doctrine.orm.cache.provider.result_cache_pool',
+                'cacheName' => 'result_cache_driver',
+                'cacheConfig' => ['type' => 'pool', 'pool' => 'result_cache_pool'],
             ],
 
             'metadata_cache_service' => [
@@ -731,29 +783,10 @@ class DoctrineExtensionTest extends TestCase
                 'cacheName' => 'result_cache_driver',
                 'cacheConfig' => ['type' => 'service', 'id' => 'service_target_result'],
             ],
-
-            'metadata_cache_array' => [
-                'expectedAliasName' => 'doctrine.orm.default_metadata_cache',
-                'expectedAliasTarget' => 'doctrine_cache.providers.doctrine.orm.default_metadata_cache',
-                'cacheName' => 'metadata_cache_driver',
-                'cacheConfig' => 'array',
-            ],
-            'query_cache_array' => [
-                'expectedAliasName' => 'doctrine.orm.default_query_cache',
-                'expectedAliasTarget' => 'doctrine_cache.providers.doctrine.orm.default_query_cache',
-                'cacheName' => 'query_cache_driver',
-                'cacheConfig' => 'array',
-            ],
-            'result_cache_array' => [
-                'expectedAliasName' => 'doctrine.orm.default_result_cache',
-                'expectedAliasTarget' => 'doctrine_cache.providers.doctrine.orm.default_result_cache',
-                'cacheName' => 'result_cache_driver',
-                'cacheConfig' => 'array',
-            ],
         ];
     }
 
-    public function testShardManager()
+    public function testShardManager() : void
     {
         $container = $this->getContainer();
         $extension = new DoctrineExtension();
@@ -777,7 +810,7 @@ class DoctrineExtensionTest extends TestCase
         $this->assertFalse($container->hasDefinition('doctrine.dbal.bar_shard_manager'));
     }
 
-    private function getContainer($bundles = 'YamlBundle', $vendor = null)
+    private function getContainer($bundles = 'YamlBundle', $vendor = null) : ContainerBuilder
     {
         $bundles = (array) $bundles;
 
@@ -788,7 +821,7 @@ class DoctrineExtensionTest extends TestCase
             $map[$bundle] = 'Fixtures\\Bundles\\' . ($vendor ? $vendor . '\\' : '') . $bundle . '\\' . $bundle;
         }
 
-        return new ContainerBuilder(new ParameterBag([
+        $container = new ContainerBuilder(new ParameterBag([
             'kernel.name' => 'app',
             'kernel.debug' => false,
             'kernel.bundles' => $map,
@@ -796,14 +829,21 @@ class DoctrineExtensionTest extends TestCase
             'kernel.environment' => 'test',
             'kernel.root_dir' => __DIR__ . '/../../', // src dir
         ]));
+
+        // Register dummy cache services so we don't have to load the FrameworkExtension
+        $container->setDefinition('cache.system', (new Definition(ArrayAdapter::class))->setPublic(true));
+        $container->setDefinition('cache.app', (new Definition(ArrayAdapter::class))->setPublic(true));
+        $container->setDefinition('my_pool', (new Definition(ArrayAdapter::class))->setPublic(true));
+
+        return $container;
     }
 
-    private function assertDICConstructorArguments(Definition $definition, array $args)
+    private function assertDICConstructorArguments(Definition $definition, array $args) : void
     {
         $this->assertEquals($args, $definition->getArguments(), "Expected and actual DIC Service constructor arguments of definition '" . $definition->getClass() . "' don't match.");
     }
 
-    private function assertDICDefinitionMethodCallAt($pos, Definition $definition, $methodName, array $params = null)
+    private function assertDICDefinitionMethodCallAt(int $pos, Definition $definition, string $methodName, array $params = null) : void
     {
         $calls = $definition->getMethodCalls();
         if (! isset($calls[$pos][0])) {
@@ -821,11 +861,8 @@ class DoctrineExtensionTest extends TestCase
 
     /**
      * Assertion for the DI Container, check if the given definition contains a method call with the given parameters.
-     *
-     * @param string     $methodName
-     * @param array|null $params
      */
-    private function assertDICDefinitionMethodCallOnce(Definition $definition, $methodName, array $params = null)
+    private function assertDICDefinitionMethodCallOnce(Definition $definition, string $methodName, array $params = null) : void
     {
         $calls  = $definition->getMethodCalls();
         $called = false;
@@ -850,7 +887,7 @@ class DoctrineExtensionTest extends TestCase
         $this->fail("Method '" . $methodName . "' is expected to be called once, definition does not contain a call though.");
     }
 
-    private function compileContainer(ContainerBuilder $container)
+    private function compileContainer(ContainerBuilder $container) : void
     {
         $container->getCompilerPassConfig()->setOptimizationPasses([new ResolveChildDefinitionsPass()]);
         $container->getCompilerPassConfig()->setRemovingPasses([]);
